@@ -16,9 +16,13 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* VTE accessibility object.  Based heavily on inspection of libzvt's
- * accessibility code. */
-
+/**
+ * SECTION: vte-access
+ * @short_description: Accessibility peer of #VteTerminal
+ *
+ * The accessibility peer of a #VteTerminal, implementing GNOME's accessibility
+ * framework.
+ */
 
 #include <config.h>
 
@@ -935,7 +939,7 @@ vte_terminal_accessible_get_text(AtkText *text,
 	gchar *ret;
 
         /* Swap around if start is greater than end */
-        if (start_offset > end_offset) {
+        if (start_offset > end_offset && end_offset != -1) {
                 gint tmp;
 
                 tmp = start_offset;
@@ -1019,10 +1023,10 @@ vte_terminal_accessible_get_text_somewhere(AtkText *text,
 			offset, priv->snapshot_attributes->len);
 	g_assert(priv->snapshot_text != NULL);
 	g_assert(priv->snapshot_characters != NULL);
-	if (offset == (int) priv->snapshot_characters->len) {
+	if (offset >= (int) priv->snapshot_characters->len) {
 		return g_strdup("");
 	}
-	g_assert(offset < priv->snapshot_characters->len);
+	g_assert(offset < (int) priv->snapshot_characters->len);
 	g_assert(offset >= 0);
 
 	switch (boundary_type) {
@@ -1302,7 +1306,7 @@ vte_terminal_accessible_get_character_at_offset(AtkText *text, gint offset)
 	priv = g_object_get_data(G_OBJECT(text),
 				 VTE_TERMINAL_ACCESSIBLE_PRIVATE_DATA);
 
-	g_assert(offset < priv->snapshot_characters->len);
+	g_assert(offset < (int) priv->snapshot_characters->len);
 
 	mapped = g_array_index(priv->snapshot_characters, int, offset);
 
@@ -1765,21 +1769,27 @@ vte_terminal_accessible_set_size(AtkComponent *component,
 				 gint width, gint height)
 {
 	VteTerminal *terminal;
-	gint columns, rows, xpad, ypad;
+	gint columns, rows, char_width, char_height;
 	GtkWidget *widget;
+        GtkBorder *inner_border;
+
 	widget = GTK_ACCESSIBLE(component)->widget;
 	if (widget == NULL) {
 		return FALSE;
 	}
 	terminal = VTE_TERMINAL(widget);
-	vte_terminal_get_padding(terminal, &xpad, &ypad);
+
+        char_width = vte_terminal_get_char_width (terminal);
+        char_height = vte_terminal_get_char_height (terminal);
+        gtk_widget_style_get (widget, "inner-border", &inner_border, NULL);
 	/* If the size is an exact multiple of the cell size, use that,
 	 * otherwise round down. */
-        columns = (width - xpad) / terminal->char_width;
-        rows = (height - ypad) / terminal->char_height;
+        columns = (width - (inner_border ? (inner_border->left + inner_border->right) : 0)) / char_width;
+        rows = (height - (inner_border ? (inner_border->top + inner_border->bottom) : 0)) / char_height;
+        gtk_border_free (inner_border);
 	vte_terminal_set_size(terminal, columns, rows);
-	return (terminal->row_count == rows) &&
-	       (terminal->column_count == columns);
+	return (vte_terminal_get_row_count (terminal) == rows) &&
+	       (vte_terminal_get_column_count (terminal) == columns);
 }
 
 
