@@ -1200,6 +1200,8 @@ vte_sequence_handler_cs (VteTerminal *terminal, GValueArray *params)
 	GValue *value;
 	VteScreen *screen;
 
+	_vte_terminal_home_cursor (terminal);
+
 	/* We require two parameters.  Anything less is a reset. */
 	screen = terminal->pvt->screen;
 	if ((params == NULL) || (params->n_values < 2)) {
@@ -1232,7 +1234,6 @@ vte_sequence_handler_cs (VteTerminal *terminal, GValueArray *params)
 	    screen->scrolling_region.end == rows - 1) {
 		screen->scrolling_restricted = FALSE;
 	}
-	_vte_terminal_home_cursor (terminal);
 }
 
 /* Restrict scrolling and updates to a subset of the visible lines, because
@@ -1558,7 +1559,7 @@ vte_sequence_handler_horizontal_and_vertical_position (VteTerminal *terminal, GV
 static void
 vte_sequence_handler_ic (VteTerminal *terminal, GValueArray *params)
 {
-	struct vte_cursor_position save;
+	VteVisualPosition save;
 	VteScreen *screen;
 
 	screen = terminal->pvt->screen;
@@ -2082,7 +2083,6 @@ vte_sequence_handler_ta (VteTerminal *terminal, GValueArray *params)
 			if (!found) {
 				VteCell *cell = _vte_row_data_get_writable (rowdata, col);
 				VteCell tab = *cell;
-				tab.attr.invisible = 1; /* FIXME: bug 499944 */
 				tab.attr.columns = newcol - col;
 				tab.c = '\t';
 				/* Check if it fits in columns */
@@ -2326,17 +2326,19 @@ vte_sequence_handler_character_attributes (VteTerminal *terminal, GValueArray *p
 			break;
 		case 38:
 		{
-			GValue *value1;
-			long param1;
 			/* The format looks like: ^[[38;5;COLORNUMBERm,
 			   so look for COLORNUMBER here. */
 			if ((i + 2) < params->n_values){
-				value1 = g_value_array_get_nth(params, i + 2);
-				if (!G_VALUE_HOLDS_LONG(value1)) {
+				GValue *value1, *value2;
+				long param1, param2;
+				value1 = g_value_array_get_nth(params, i + 1);
+				value2 = g_value_array_get_nth(params, i + 2);
+				if (G_UNLIKELY (!(G_VALUE_HOLDS_LONG(value1) && G_VALUE_HOLDS_LONG(value2))))
 					break;
-				}
 				param1 = g_value_get_long(value1);
-				terminal->pvt->screen->defaults.attr.fore = param1;
+				param2 = g_value_get_long(value2);
+				if (G_LIKELY (param1 == 5 && param2 >= 0 && param2 < 256))
+					terminal->pvt->screen->defaults.attr.fore = param2;
 				i += 2;
 			}
 			break;
@@ -2360,17 +2362,19 @@ vte_sequence_handler_character_attributes (VteTerminal *terminal, GValueArray *p
 			break;
 		case 48:
 		{
-			GValue *value1;
-			long param1;
 			/* The format looks like: ^[[48;5;COLORNUMBERm,
 			   so look for COLORNUMBER here. */
 			if ((i + 2) < params->n_values){
-				value1 = g_value_array_get_nth(params, i + 2);
-				if (!G_VALUE_HOLDS_LONG(value1)) {
+				GValue *value1, *value2;
+				long param1, param2;
+				value1 = g_value_array_get_nth(params, i + 1);
+				value2 = g_value_array_get_nth(params, i + 2);
+				if (G_UNLIKELY (!(G_VALUE_HOLDS_LONG(value1) && G_VALUE_HOLDS_LONG(value2))))
 					break;
-				}
 				param1 = g_value_get_long(value1);
-				terminal->pvt->screen->defaults.attr.back = param1;
+				param2 = g_value_get_long(value2);
+				if (G_LIKELY (param1 == 5 && param2 >= 0 && param2 < 256))
+					terminal->pvt->screen->defaults.attr.back = param2;
 				i += 2;
 			}
 			break;
@@ -2523,6 +2527,32 @@ vte_sequence_handler_set_scrolling_region (VteTerminal *terminal, GValueArray *p
 	vte_sequence_handler_offset(terminal, params, -1, vte_sequence_handler_cs);
 }
 
+static void
+vte_sequence_handler_set_scrolling_region_from_start (VteTerminal *terminal, GValueArray *params)
+{
+	GValue value = {0};
+
+	g_value_init (&value, G_TYPE_LONG);
+	g_value_set_long (&value, 0); /* Out of range means start/end */
+
+	g_value_array_insert (params, 0, &value);
+
+	vte_sequence_handler_offset(terminal, params, -1, vte_sequence_handler_cs);
+}
+
+static void
+vte_sequence_handler_set_scrolling_region_to_end (VteTerminal *terminal, GValueArray *params)
+{
+	GValue value = {0};
+
+	g_value_init (&value, G_TYPE_LONG);
+	g_value_set_long (&value, 0); /* Out of range means start/end */
+
+	g_value_array_insert (params, 1, &value);
+
+	vte_sequence_handler_offset(terminal, params, -1, vte_sequence_handler_cs);
+}
+
 /* Set the application or normal keypad. */
 static void
 vte_sequence_handler_application_keypad (VteTerminal *terminal, GValueArray *params)
@@ -3183,29 +3213,21 @@ vte_sequence_handler_window_manipulation (VteTerminal *terminal, GValueArray *pa
 			vte_terminal_feed_child(terminal, buf, -1);
 			break;
 		case 20:
-			/* Report a static icon title, since the real
-			   icon title should NEVER be reported, as it
-			   creates a security vulnerability.  See
-			   http://marc.info/?l=bugtraq&m=104612710031920&w=2
-			   and CVE-2003-0070. */
+			/* Report the icon title. */
 			_vte_debug_print(VTE_DEBUG_PARSE,
-				"Reporting fake icon title.\n");
-			/* never use terminal->icon_title here! */
+				"Reporting icon title.\n");
 			g_snprintf (buf, sizeof (buf),
-				    _VTE_CAP_OSC "LTerminal" _VTE_CAP_ST);
+				    _VTE_CAP_OSC "L%s" _VTE_CAP_ST,
+				    terminal->icon_title);
 			vte_terminal_feed_child(terminal, buf, -1);
 			break;
 		case 21:
-			/* Report a static window title, since the real
-			   window title should NEVER be reported, as it
-			   creates a security vulnerability.  See
-			   http://marc.info/?l=bugtraq&m=104612710031920&w=2
-			   and CVE-2003-0070. */
+			/* Report the window title. */
 			_vte_debug_print(VTE_DEBUG_PARSE,
-					"Reporting fake window title.\n");
-			/* never use terminal->window_title here! */
+					"Reporting window title.\n");
 			g_snprintf (buf, sizeof (buf),
-				    _VTE_CAP_OSC "lTerminal" _VTE_CAP_ST);
+				    _VTE_CAP_OSC "l%s" _VTE_CAP_ST,
+				    terminal->window_title);
 			vte_terminal_feed_child(terminal, buf, -1);
 			break;
 		default:
